unit Video;
{ Text-mode videuo unit v1.00      Copyright (c) 1997, Marius Gedminas }
{                                                                      }
{ It gives you TurboVision like fast screen output. Supports ALL text  }
{ modes if their base address is B800:0000 (except mode 7 - B000:0000) }
{                                                                      }
{ Don't forget to call InitVideo after (possible) mode change (possible}
{ - e.g. after calling some other DOS program)!                        }
{                                                                      }
{ This unit is free non-commercial use. You may change the code below  }
{ for your own use, but don't give away modified versions of it. If    }
{ you think you made an improvement, please contact me at              }
{ mgedmin@pub.osf.lt                                                   }
{                                                                      }
{ Warning: this unit wasn't tested very thoroughly. It may contain     }
{ bugs.                                                                }
{                                                                      }
{ The code is *very* poorly commented :-(                              }
{                                                                      }
{ A short disclaimer: use this at your own risk. I won't be liable for }
{ ANYTHING concerning this unit, and I don't give ANY warranties.      }


{$G+ 286 opcodes }

interface

  const
    MaxScrX = 132;
    MaxScrY = 100;
    MaxScrSize = MaxScrX * MaxScrY;

  type
    WordRec = record
      Lo, Hi: Byte;
    end;
    LongRec = record
      Lo, Hi: Word;
    end;
    TPoint = record
      X, Y: Integer;
    end;
    ScrRec = record
      C: Char;
      A: Byte;
    end;

  type
    PScrBuf = ^TScrBuf;
    TScrBuf = array[0..MaxScrSize] of Word;
    TDrawBuffer = array[0..MaxScrX-1] of Word;

  var
    Screen: Pointer;
    ScrW: Word;
    ScrH: Word;
    ScrSize: Word;
    ScrMode: Byte;
    Aspect: Byte; { 1 for 40x25 or 80x50, 2 for 80x25 }

  procedure InitVideo;
  procedure ClearScreen;
  procedure FillScreen(C: Char; A: Byte);
  procedure Blink(On: Boolean);

  procedure FillAttr(x, y, w, h: Integer; A: Byte);

  procedure WriteChar(x, y: Integer; C: Char; A: Byte; Count: Integer);
  procedure WriteStr(x, y: Integer; const S: string; A: Byte);
  procedure WriteCStr(x, y: Integer; const S: string; A: Word);
  procedure WriteLine(x, y, w, h: Integer; const Buf);
  procedure WriteBuf(x, y, w, h: Integer; const Buf);

  procedure SaveBuf(x, y, w, h: Integer; var Buf);

  procedure MoveBuf(var Dest; const Source; Attr: Byte; Count: Word);
  procedure MoveChar(var Dest; C: Char; Attr: Byte; Count: Word);
  procedure MoveCStr(var Dest; const Str: string; Attrs: Word);
  procedure MoveStr(var Dest; const Str: string; Attr: Byte);

  function CStrLen(const S: string): Integer;

  const
    NoCursor  = $2000;
    StdCursor = $0607;
    BigCursor = $0007;

  procedure SetCursor(Shape: Word);
  function GetCursor: Word;

  procedure SetCursorPos(X, Y: Byte);
  procedure GetCursorPos(var X, Y: Byte);

implementation

procedure InitVideo;
begin
  asm
    mov  ah, 0Fh
    int  10h
    mov  ScrMode, al
    shr  ax, 8
    mov  ScrW, ax
  end;
  asm
    mov  ax, 1130h
    mov  bh, 0
    mov  dl, 24
    push bp
    int  10h
    pop  bp
    mov  dh, 0
    inc  dx
    mov  ScrH, dx
  end;
  if ScrMode = 7 then Screen := Ptr(SegB000,0)
  else Screen := Ptr(SegB800,0);
  ScrSize := ScrW*ScrH;
  Aspect := (ScrW + ScrH) div (ScrH shl 1);
  if Aspect = 0 then Aspect := 1;
end;

procedure ClearScreen; assembler;
asm
  les  di, Screen
  mov  cx, ScrSize
  mov  ax, 0720h
  cld
  rep stosw
end;

procedure FillScreen(C: Char; A: Byte); assembler;
asm
  les  di, Screen
  mov  cx, ScrSize
  mov  al, C
  mov  ah, A
  cld
  rep stosw
end;

procedure Blink(On: Boolean); assembler;
asm
  mov  ax, 1003h
  mov  bl, On
  int  10h
end;


procedure FillAttr(x, y, w, h: Integer; A: Byte); assembler;
asm
  cmp  w, 0
  jle  @@2
  cmp  h, 0
  jle  @@2

  cld
  les  di, Screen
  mov  ax, ScrW
  mul  y
  add  ax, x
  add  ax, ax
  add  di, ax

  mov  dx, ScrW
  sub  dx, w
  add  dx, dx

  mov  al, A
  mov  bx, h
@@1:
  mov  cx, w
@@3:
  inc  di
  stosb
  dec  cx
  jnz  @@3
  add  di, dx
  dec  bx
  jnz  @@1

@@2:
end;


procedure WriteChar(x, y: Integer; C: Char; A: Byte; Count: Integer); assembler;
asm
  mov  cx, Count
  or   cx, cx
  jle  @@1

  les  di, Screen
  mov  ax, ScrW
  mul  y
  add  ax, x
  add  ax, ax
  add  di, ax

  mov  al, C
  mov  ah, A
  cld
  rep stosw

@@1:
end;

procedure WriteStr(x, y: Integer; const S: string; A: Byte); assembler;
asm
  push ds

  les  di, Screen
  mov  ax, ScrW
  mul  y
  add  ax, x
  add  ax, ax
  add  di, ax

  lds  si, S
  cld
  lodsb
  xor  cx, cx
  add  cl, al
  jz   @@2

  mov  ah, A
@@1:
  lodsb
  stosw
  loop @@1

@@2:
  pop  ds
end;

procedure WriteCStr(x, y: Integer; const S: string; A: Word); assembler;
asm
  push ds

  les  di, Screen
  mov  ax, ScrW
  mul  y
  add  ax, x
  add  ax, ax
  add  di, ax

  lds  si, S
  cld
  lodsb
  xor  cx, cx
  add  cl, al
  jz   @@3

  mov  bx, A
  mov  ah, bl
@@1:
  lodsb
  cmp  al, '~'
  je   @@2
  stosw
  loop @@1
  jmp  @@3
@@2:
  xchg bl, bh
  mov  ah, bl
  loop @@1

@@3:
  pop  ds
end;

procedure WriteLine(x, y, w, h: Integer; const Buf); assembler;
asm
  cmp  w, 0
  jle  @@2
  cmp  h, 0
  jle  @@2

  push ds

  cld
  les  di, Screen
  mov  ax, ScrW
  mul  y
  add  ax, x
  add  ax, ax
  add  di, ax

  mov  ax, ScrW
  sub  ax, w
  add  ax, ax

  mov  bx, h
@@1:
  lds  si, Buf
  mov  cx, w
  rep movsw
  add  di, ax
  dec  bx
  jnz  @@1

  pop  ds

@@2:
end;

procedure WriteBuf(x, y, w, h: Integer; const Buf); assembler;
asm
  cmp  w, 0
  jle  @@2
  cmp  h, 0
  jle  @@2

  push ds

  cld
  les  di, Screen
  mov  ax, ScrW
  mul  y
  add  ax, x
  add  ax, ax
  add  di, ax

  mov  ax, ScrW
  sub  ax, w
  add  ax, ax

  lds  si, Buf

  mov  bx, h
@@1:
  mov  cx, w
  rep movsw
  add  di, ax
  dec  bx
  jnz  @@1

  pop  ds

@@2:
end;


procedure SaveBuf(x, y, w, h: Integer; var Buf); assembler;
asm
  cmp  w, 0
  jle  @@2
  cmp  h, 0
  jle  @@2

  push ds

  cld

  les  di, Buf

  mov  bx, ScrW

  lds  si, Screen
  mov  ax, bx
  mul  y
  add  ax, x
  add  ax, ax
  add  si, ax

  mov  ax, bx
  sub  ax, w
  add  ax, ax

  mov  bx, h
@@1:
  mov  cx, w
  rep movsw
  add  si, ax
  dec  bx
  jnz  @@1

  pop  ds

@@2:
end;


procedure MoveBuf(var Dest; const Source; Attr: Byte; Count: Word); assembler;
asm
	MOV	CX,Count
	JCXZ	@@5
	MOV	DX,DS
	LES	DI,Dest
	LDS	SI,Source
	MOV	AH,Attr
	CLD
	OR	AH,AH
	JE	@@3
@@1:	LODSB
	STOSW
	LOOP	@@1
	JMP	@@4
@@2:	INC	DI
@@3:	MOVSB
	LOOP	@@2
@@4:	MOV	DS,DX
@@5:
end;

procedure MoveChar(var Dest; C: Char; Attr: Byte; Count: Word); assembler;
asm
	MOV	CX,Count
	JCXZ	@@4
	LES	DI,Dest
	MOV	AL,C
	MOV	AH,Attr
	CLD
	OR	AL,AL
	JE	@@1
	OR	AH,AH
	JE	@@3
	REP	STOSW
	JMP	@@4
@@1:	MOV	AL,AH
@@2:	INC	DI
@@3:	STOSB
	LOOP	@@2
@@4:
end;

procedure MoveCStr(var Dest; const Str: String; Attrs: Word); assembler;
asm
	MOV	DX,DS
	LDS	SI,Str
	CLD
	LODSB
	MOV	CL,AL
	XOR	CH,CH
	JCXZ	@@3
	LES	DI,Dest
	MOV	BX,Attrs
	MOV	AH,BL
@@1:	LODSB
	CMP	AL,'~'
	JE	@@2
	STOSW
	LOOP	@@1
	JMP	@@3
@@2:	XCHG	AH,BH
	LOOP	@@1
@@3:	MOV	DS,DX
end;

procedure MoveStr(var Dest; const Str: String; Attr: Byte); assembler;
asm
	MOV	DX,DS
	LDS	SI,Str
	CLD
	LODSB
	MOV	CL,AL
	XOR	CH,CH
	JCXZ	@@4
	LES	DI,Dest
	MOV	AH,Attr
	OR	AH,AH
	JE	@@3
@@1:	LODSB
	STOSW
	LOOP	@@1
	JMP	@@4
@@2:	INC	DI
@@3:	MOVSB
	LOOP	@@2
@@4:	MOV	DS,DX
end;


function CStrLen(const S: String): Integer; assembler;
asm
	LES	DI,S
	MOV	CL,ES:[DI]
	INC	DI
	XOR	CH,CH
	MOV	BX,CX
        JCXZ    @@2
	MOV	AL,'~'
        CLD
@@1:	REPNE	SCASB
	JNE	@@2
	DEC	BX
	JMP	@@1
@@2:	MOV	AX,BX
end;

  { - Cursor - }

procedure SetCursor(Shape: Word); assembler;
asm
  mov  cx, Shape
  mov  ah, 1
  int  10h
end;

function GetCursor: Word; assembler;
asm
  mov  ah, 3
  mov  bh, 0
  int  10h
  mov  ax, cx
end;

procedure SetCursorPos(X, Y: Byte); assembler;
asm
  mov  ah, 2
  mov  bh, 0
  mov  dh, Y
  mov  dl, X
  int  10h
end;

procedure GetCursorPos(var X, Y: Byte); assembler;
asm
  mov  ah, 3
  mov  bh, 0
  int  10h
  les  bx, X
  mov  es:[bx], dl
  les  bx, Y
  mov  es:[bx], dh
end;

begin
  InitVideo;
end.